@@ -1,6 +1,7 @@
-from .utils.fflow import init, gen_task, gen_task_from_para, gen_benchmark_from_file, gen_decentralized_benchmark, gen_hierarchical_benchmark, convert_model,tune, run_in_parallel, multi_init_and_run
-
+from .utils.fflow import init, gen_task, gen_task_by_, gen_benchmark_from_file, gen_decentralized_benchmark, gen_hierarchical_benchmark, convert_model,tune, run_in_parallel, module2fmodule, multi_init_and_run, set_data_root,download_resource, list_resource, option_helper
+from .benchmark import data_root
 communicator = None
+_data_root = data_root
 
 class VirtualCommunicator:
     """
