@@ -7,6 +7,7 @@ import torch
 import torch.multiprocessing as mp
 import numpy as np
 
+import flgo.benchmark.base
 from flgo.utils import fmodule
 import flgo.simulator.base as ss
 
@@ -193,6 +194,7 @@ class BasicParty:
         return
 
 class BasicServer(BasicParty):
+    TaskCalculator = flgo.benchmark.base.BasicTaskCalculator
     def __init__(self, option={}):
         super().__init__()
         self.test_data = None
@@ -206,7 +208,7 @@ class BasicServer(BasicParty):
         self.num_parallels = option['num_parallels']
         # server calculator
         self.device = self.gv.apply_for_device() if not option['server_with_cpu'] else torch.device('cpu')
-        self.calculator = self.gv.TaskCalculator(self.device, optimizer_name=option['optimizer'])
+        self.calculator = self.TaskCalculator(self.device, optimizer_name=option['optimizer'])
         # hyper-parameters during training process
         self.num_rounds = option['num_rounds']
         self.num_steps = option['num_steps']
@@ -462,6 +464,17 @@ class BasicServer(BasicParty):
         ```
         """
         if len(models) == 0: return self.model
+        nan_exists = [m.has_nan() for m in models]
+        if any(nan_exists):
+            self.gv.logger.info('Warning("There exists non-value in local models")')
+            new_models = []
+            received_clients = []
+            for ni, mi, cid in zip(nan_exists, models, self.received_clients):
+                if ni: continue
+                new_models.append(mi)
+                received_clients.append(cid)
+            self.received_clients = received_clients
+            models = new_models
         local_data_vols = [c.datavol for c in self.clients]
         total_data_vol = sum(local_data_vols)
         if self.aggregation_option == 'weighted_scale':
@@ -569,6 +582,15 @@ class BasicServer(BasicParty):
         """
         return self.tolerance_for_latency
 
+    def set_tolerance_for_latency(self, tolerance:int):
+        r"""
+        Set the tolerance for latency of waiting for clients' responses
+
+        Args:
+            tolerance (int): the amounts of virtual time units
+        """
+        self.tolerance_for_latency = tolerance
+
     def wait_time(self, t=1):
         r"""
         Wait for the time of the virtual clock to pass t units
@@ -603,6 +625,7 @@ class BasicServer(BasicParty):
         self.dropped_clients = []
 
 class BasicClient(BasicParty):
+    TaskCalculator = flgo.benchmark.base.BasicTaskCalculator
     def __init__(self, option={}):
         super().__init__()
         self.id = None
@@ -614,7 +637,7 @@ class BasicClient(BasicParty):
         self.model = None
         # local_movielens_recommendation calculator
         self.device = self.gv.apply_for_device()
-        self.calculator = self.gv.TaskCalculator(self.device, option['optimizer'])
+        self.calculator = self.TaskCalculator(self.device, option['optimizer'])
         self._train_loader = None
         # hyper-parameters for training
         self.optimizer_name = option['optimizer']
