@@ -10,6 +10,8 @@ from flgo.utils import fmodule
 class Server(BasicServer):
     def initialize(self, *args, **kwargs):
         self.init_algo_para({'mu':0.1})
+        self.sample_option = 'md'
+        self.aggregation_option = 'uniform'
 
 class Client(BasicClient):
     @fmodule.with_multi_gpus
