@@ -1,3 +1,6 @@
+"""This is a non-official implementation of 'Fair Resource Allocation in
+Federated Learning' (http://arxiv.org/abs/1905.10497).  And this implementation
+refers to the official github repository https://github.com/litian96/fair_flearn """
 import flgo.algorithm.fedbase as fedbase
 import flgo.utils.fmodule as fmodule
 import copy
@@ -25,4 +28,4 @@ class Client(fedbase.BasicClient):
         dk = (Fk ** self.q) * delta_wk
         hk = self.q * (Fk ** (self.q - 1)) * (delta_wk.norm() ** 2) + L * (Fk ** self.q)
         self.global_model = None
-        return {'dk': dk, 'hk': hk}
\ No newline at end of file
+        return {'dk': dk, 'hk': hk}
