@@ -304,7 +304,12 @@ class BasicTaskPipe(AbstractTaskPipe):
         if p == 0: return dataset, None
         s1 = int(len(dataset) * p)
         s2 = len(dataset) - s1
-        return torch.utils.data.random_split(dataset, [s2, s1])
+        if s1==0:
+            return dataset, None
+        elif s2==0:
+            return None, dataset
+        else:
+            return torch.utils.data.random_split(dataset, [s2, s1])
 
     def task_exists(self):
         r"""
@@ -361,7 +366,7 @@ class BasicTaskCalculator(AbstractTaskCalculator):
     def to_device(self, data, *args, **kwargs):
         return NotImplementedError
 
-    def get_dataloader(self, *args, **kwargs):
+    def get_dataloader(self, dataset, batch_size=64, *args, **kwargs):
         return NotImplementedError
 
     def test(self, model, data, *args, **kwargs):
