@@ -19,7 +19,7 @@ class GCN(torch.nn.Module):
         x = self.conv2(x, edge_index)
         return F.log_softmax(x, dim=1)
 
-path = os.path.join(flgo.benchmark.path,'RAW_DATA', 'CITESEER')
+path = os.path.join(flgo.benchmark.data_root, 'CITESEER')
 dataset = torch_geometric.datasets.Planetoid(path, name='Citeseer')
 train_data = dataset[0]
 
