@@ -5,7 +5,7 @@ import torch.nn
 from torchtext.vocab import build_vocab_from_iterator
 from torchtext.data.utils import get_tokenizer, ngrams_iterator
 
-path = os.path.join(flgo.benchmark.path, 'RAW_DATA','IMDB')
+path = os.path.join(flgo.benchmark.data_root, 'IMDB')
 train_data = torchtext.datasets.IMDB(root=path, split='train')
 test_data = torchtext.datasets.IMDB(root=path, split='test')
 ngrams = 2
@@ -17,6 +17,18 @@ def yield_tokens(data_iter, ngrams):
 vocab = build_vocab_from_iterator(yield_tokens(train_data, ngrams), specials=["<unk>"])
 vocab.set_default_index(vocab["<unk>"])
 
+def text_pipeline(x):
+    return vocab(list(ngrams_iterator(tokenizer(x), ngrams)))
+
+def label_pipeline(x):
+    return int(x) - 1
+
+def apply_transform(x):
+    return text_pipeline(x[1]), label_pipeline(x[0])
+
+train_data = train_data.map(apply_transform)
+test_data = test_data.map(apply_transform)
+
 class TextClassificationModel(torch.nn.Module):
     def __init__(self, vocab_size, embed_dim, num_class):
         super(TextClassificationModel, self).__init__()
@@ -30,7 +42,13 @@ class TextClassificationModel(torch.nn.Module):
         self.fc.weight.data.uniform_(-initrange, initrange)
         self.fc.bias.data.zero_()
 
-    def forward(self, text, offsets):
+    def forward(self, text):
+        offsets = [0]
+        for t in text:
+            offsets.append(t.size(0))
+        offsets = torch.tensor(offsets[:-1]).cumsum(dim=0)
+        text = torch.cat(text)
+        offsets = offsets.to(text.device)
         embedded = self.embedding(text, offsets)
         return self.fc(embedded)
 
