@@ -7,22 +7,22 @@ from flgo.utils.fmodule import FModule
 class Model(FModule):
     def __init__(self):
         super(Model, self).__init__()
-        self.fc1 = nn.Linear(784, 200)
-        self.fc2 = nn.Linear(200, 200)
-        self.fc3 = nn.Linear(200, 10)
+        self.encoder = nn.Sequential(
+            nn.Flatten(1),
+            nn.Linear(784, 200),
+            nn.ReLU(),
+            nn.Linear(200, 200),
+            nn.ReLU()
+        )
+        self.head = nn.Linear(200, 10)
+
 
     def forward(self, x):
-        x = self.get_embedding(x)
-        x = self.fc3(x)
+        x = self.encoder(x)
+        x = self.head(x)
         return x
 
-    def get_embedding(self, x):
-        x = x.view(-1, x.shape[1] * x.shape[-2] * x.shape[-1])
-        x = self.fc1(x)
-        x = F.relu(x)
-        x = self.fc2(x)
-        x = F.relu(x)
-        return x
+
 
 def init_local_module(object):
     pass
