@@ -2165,6 +2165,7 @@ class BasicLogger(Logger):
             self.addHandler(self.filehandler)
         # options of early stopping
         self._es_key = 'val_loss'
+        self._es_direction = -1
         self._es_patience = 20
         self._es_counter = 0
         self._es_best_score = None
@@ -2270,18 +2271,22 @@ class BasicLogger(Logger):
         else:
             output_name = output_name + ("K{}_".format(self.option['num_steps']))
 
-        output_name = output_name + "LR{:.4f}_P{:.2f}_S{}_LD{:.3f}_WD{:.3f}_AVL{}_CN{}_CP{}_RS{}_LG{}".format(
+        output_name = output_name + "LR{:.4f}_P{:.2f}_S{}_LD{:.3f}_WD{:.3f}".format(
                         self.option['learning_rate'],
                         self.option['proportion'],
                         self.option['seed'],
                         self.option['lr_scheduler'] + self.option['learning_rate_decay'],
                         self.option['weight_decay'],
+        )
+        output_name = output_name + '_SIM{}_AVL{}_CN{}_CP{}_RS{}_LG{}'.format(
+                        self.simulator if hasattr(self, 'simulator') else 'None',
                         self.option['availability'],
                         self.option['connectivity'],
                         self.option['completeness'],
                         self.option['responsiveness'],
                         self.__class__.__name__,
-        ) + suffix
+        )
+        output_name = output_name + suffix
         return output_name
 
     def get_output_path(self):
@@ -2294,10 +2299,18 @@ class BasicLogger(Logger):
     def get_time_string(self):
         return time.strftime('%Y-%m-%d-%H-%M-%S')
 
+    def turn_early_stop_direction(self):
+        """
+        _es_direction=1 infers that a higher value of the validation metric is better and _es_direction=-1 means lower is better.
+        The default value of _es_direction is -1.
+        Returns:
+        """
+        self._es_direction = -self._es_direction
+
     def early_stop(self):
         # Early stopping when there is no improvement on the validation loss for more than self.option['early_stop'] rounds
         if self.option['early_stop']<0 or (self._es_key not in self.output): return False
-        score = -self.output[self._es_key][-1]
+        score = self._es_direction*self.output[self._es_key][-1]
         if np.isnan(score): return True
         if self._es_best_score is None:
             self._es_best_score = score
