@@ -6,6 +6,9 @@ import functools
 import torch
 import heapq
 
+import flgo.utils.fmodule
+
+
 class PriorityQueue:
     r"""Priority Queue"""
     def __init__(self):
@@ -53,9 +56,10 @@ def size_of_package(package):
         size (int): the size of the package
     """
     size = 0
+    if not isinstance(package, dict): return 0
     for v in package.values():
         if type(v) is torch.Tensor:
-            size += sys.getsizeof(v.storage())
+            size += sys.getsizeof(v.untyped_storage())
         else:
             size += v.__sizeof__()
     return size
@@ -218,7 +222,7 @@ class BasicSimulator(AbstractSimulator):
             self.clients = {c.id:c for c in objects[1:]}
         else:
             self.server = None
-            self.clients = []
+            self.clients = {}
         self.all_clients = list(self.clients.keys())
         self.random_module = np.random.RandomState(0)
         # client states and the variables
@@ -236,6 +240,9 @@ class BasicSimulator(AbstractSimulator):
             self.set_variable(self.all_clients, var, [self.variables[cid][var] for cid in self.all_clients])
         self.state_counter = {c:{'dropped_counter': 0, 'latency_counter': 0, } for c in self.clients}
 
+    def initialize(self, *args, **kwargs):
+        return
+
     def get_client_with_state(self, state='idle'):
         r"""
         Get clients according to their states.
@@ -497,11 +504,15 @@ def with_latency(communicate_with):
     def delayed_communicate_with(self, target_id, package):
         # Calculate latency for the target client
         # Set local_movielens_recommendation model size of clients for computation cost estimation
-        model_size = package['model'].count_parameters(output=False) if 'model' in package.keys() else 0
+        if 'model' in package.keys() and isinstance(package['model'], flgo.utils.fmodule.FModule):
+            model_size = package['model'].count_parameters(output=False)
+        else:
+            model_size = 0
         self.gv.simulator.set_variable(target_id, '__model_size', model_size)
         # Set downloading package sizes for clients for downloading cost estimation
         self.gv.simulator.set_variable(target_id, '__download_package_size',size_of_package(package))
         res = communicate_with(self, target_id, package)
+        if res is None: res = {}
         # Set uploading package sizes for clients for uploading cost estimation
         self.gv.simulator.set_variable(target_id, '__upload_package_size', size_of_package(res))
         # update latency of the target client according to the communication cost and computation cost
@@ -567,7 +578,7 @@ def with_clock(communicate):
             return res
         # Convert the unpacked packages to a list of packages of each client.
         pkgs = [{key: vi[id] for key, vi in res.items()} for id in range(len(list(res.values())[0]))] if len(selected_clients)>0 else []
-        if pkgs[0].get('__cid', None) is None:
+        if len(pkgs)>0 and pkgs[0].get('__cid', None) is None:
             for cid, pkg in zip(selected_clients, pkgs):
                 pkg['__cid'] = cid
         # Put the packages from selected clients into clock only if when there are effective selected clients
@@ -591,7 +602,7 @@ def with_clock(communicate):
             delta_t = tolerance_for_latency if any_drop or any_overdue else max_latency
             # Receive packages within due
             eff_pkgs = self.gv.clock.get_until(self.gv.clock.current_time + delta_t)
-            self.gv.clock.step(delta_t)
+            self.gv.clock.step(int(delta_t))
             # Drop the packages of overdue clients and reset their states to `idle`
             eff_cids = [pkg_i['__cid'] for pkg_i in eff_pkgs]
             self._overdue_clients = list(set([cid for cid in selected_clients if cid not in eff_cids]))
