@@ -10,7 +10,12 @@ import importlib
 import random
 import os
 import os.path
+import uuid
 import warnings
+import requests
+import re
+import zipfile
+import urllib.request
 from typing import *
 try:
     import ujson as json
@@ -40,6 +45,7 @@ import flgo.experiment.logger.hier_logger
 import flgo.experiment.logger
 import flgo.experiment.device_scheduler
 from flgo.simulator.base import BasicSimulator
+import flgo.benchmark.base
 import flgo.benchmark.partition
 import flgo.benchmark.toolkits.partition
 import flgo.algorithm
@@ -47,19 +53,19 @@ import flgo.algorithm
 sample_list=['uniform', 'md', 'full', 'uniform_available', 'md_available', 'full_available'] # sampling options for the default sampling method in flgo.algorihtm.fedbase
 agg_list=['uniform', 'weighted_scale', 'weighted_com'] # aggregation options for the default aggregating method in flgo.algorihtm.fedbase
 optimizer_list=['SGD', 'Adam', 'RMSprop', 'Adagrad'] # supported optimizers
-default_option_dict = {'pretrain': '', 'sample': 'md', 'aggregate': 'uniform', 'num_rounds': 20, 'proportion': 0.2, 'learning_rate_decay': 0.998, 'lr_scheduler': -1, 'early_stop': -1, 'num_epochs': 5, 'num_steps': -1, 'learning_rate': 0.1, 'batch_size': 64.0, 'optimizer': 'SGD', 'clip_grad':0.0,'momentum': 0.0, 'weight_decay': 0.0, 'num_edge_rounds':5, 'algo_para': [], 'train_holdout': 0.1, 'test_holdout': 0.0, 'local_test':False,'seed': 0, 'gpu': [], 'server_with_cpu': False, 'num_parallels': 1, 'num_workers': 0, 'pin_memory':False,'test_batch_size': 512,'pin_memory':False ,'simulator': 'default_simulator', 'availability': 'IDL', 'connectivity': 'IDL', 'completeness': 'IDL', 'responsiveness': 'IDL', 'logger': 'basic_logger', 'log_level': 'INFO', 'log_file': False, 'no_log_console': False, 'no_overwrite': False, 'eval_interval': 1}
+default_option_dict = {'pretrain': '', 'sample': 'md', 'aggregate': 'uniform', 'num_rounds': 20, 'proportion': 0.2, 'learning_rate_decay': 0.998, 'lr_scheduler': -1, 'early_stop': -1, 'num_epochs': 5, 'num_steps': -1, 'learning_rate': 0.1, 'batch_size': 64.0, 'optimizer': 'SGD', 'clip_grad':0.0,'momentum': 0.0, 'weight_decay': 0.0, 'num_edge_rounds':5, 'algo_para': [], 'train_holdout': 0.1, 'test_holdout': 0.0, 'local_test':False,'seed': 0,'dataseed':0, 'gpu': [], 'server_with_cpu': False, 'num_parallels': 1, 'num_workers': 0, 'pin_memory':False,'test_batch_size': 512,'pin_memory':False ,'simulator': 'default_simulator', 'availability': 'IDL', 'connectivity': 'IDL', 'completeness': 'IDL', 'responsiveness': 'IDL', 'logger': 'basic_logger', 'log_level': 'INFO', 'log_file': False, 'no_log_console': False, 'no_overwrite': False, 'eval_interval': 1}
 
 class GlobalVariable:
     """This class is to create a shared space for sharing variables across
     different parties for each runner"""
 
-    def __init__(self):
-        self.logger = None
-        self.simulator = None
-        self.clock = None
-        self.dev_list = None
-        self.TaskCalculator = None
-        self.TaskPipe = None
+    def __init__(self, logger:flgo.experiment.logger.BasicLogger=None, simulator:flgo.simulator.base.BasicSimulator=None, clock:flgo.simulator.base.ElemClock=None, dev_list:list=None, TaskCalculator:flgo.benchmark.base.BasicTaskCalculator=None, TaskPipe:flgo.benchmark.base.BasicTaskPipe=None):
+        self.logger = logger
+        self.simulator = simulator
+        self.clock = clock
+        self.dev_list = dev_list
+        self.TaskCalculator = TaskCalculator
+        self.TaskPipe = TaskPipe
         self.crt_dev = 0
 
     def apply_for_device(self):
@@ -105,8 +111,8 @@ def read_option_from_command():
     """Training Options"""
     # basic settings
     # methods of server side for sampling and aggregating
-    parser.add_argument('--sample', help='methods for sampling clients', type=str, choices=sample_list, default='md')
-    parser.add_argument('--aggregate', help='methods for aggregating models', type=str, choices=agg_list, default='uniform')
+    parser.add_argument('--sample', help='methods for sampling clients', type=str, choices=sample_list, default='uniform')
+    parser.add_argument('--aggregate', help='methods for aggregating models', type=str, choices=agg_list, default='other')
     # hyper-parameters of training in server side
     parser.add_argument('--num_rounds', help='number of communication rounds', type=int, default=20)
     parser.add_argument('--proportion', help='proportion of clients sampled per round', type=float, default=0.2)
@@ -114,27 +120,28 @@ def read_option_from_command():
     parser.add_argument('--lr_scheduler', help='type of the global learning rate scheduler', type=int, default=-1)
     parser.add_argument('--early_stop', help='stop training if there is no improvement for no smaller than the maximum rounds', type=int, default=-1)
     # hyper-parameters of local_movielens_recommendation training
-    parser.add_argument('--num_epochs', help='number of epochs when clients trainset on data;', type=int, default=5)
-    parser.add_argument('--num_steps', help='the number of local_movielens_recommendation steps, which dominate num_epochs when setting num_steps>0', type=int, default=-1)
+    parser.add_argument('--num_epochs', help='number of epochs when clients locally train the model on data;', type=int, default=5)
+    parser.add_argument('--num_steps', help='the number of local steps, which dominate num_epochs when setting num_steps>0', type=int, default=-1)
     parser.add_argument('--learning_rate', help='learning rate for inner solver;', type=float, default=0.1)
-    parser.add_argument('--batch_size', help='batch size when clients trainset on data;', type=float, default='64')
+    parser.add_argument('--batch_size', help='batch size', type=float, default='64')
     parser.add_argument('--optimizer', help='select the optimizer for gd', type=str, choices=optimizer_list, default='SGD')
     parser.add_argument('--clip_grad', help='clipping gradients if the max norm of gradients ||g|| > clip_norm > 0', type=float, default=0.0)
-    parser.add_argument('--momentum', help='momentum of local_movielens_recommendation update', type=float, default=0.0)
-    parser.add_argument('--weight_decay', help='weight decay for the training process', type=float, default=0.0)
+    parser.add_argument('--momentum', help='momentum of local training', type=float, default=0.0)
+    parser.add_argument('--weight_decay', help='weight decay of local training', type=float, default=0.0)
     parser.add_argument('--num_edge_rounds', help='number of edge rounds in hierFL', type=int, default=5)
     # algorithm-dependent hyper-parameters
     parser.add_argument('--algo_para', help='algorithm-dependent hyper-parameters', nargs='*', type=float)
 
     """Environment Options"""
     # the ratio of the amount of the data used to train
-    parser.add_argument('--train_holdout', help='the rate of holding out the validation dataset from all the local_movielens_recommendation training datasets', type=float, default=0.1)
-    parser.add_argument('--test_holdout', help='the rate of holding out the validation dataset from the training datasets', type=float, default=0.0)
+    parser.add_argument('--train_holdout', help='the rate of holding out the validation dataset from all the local training datasets', type=float, default=0.1)
+    parser.add_argument('--test_holdout', help='the rate of holding out the validation dataset from the testing datasets owned by the server', type=float, default=0.0)
     parser.add_argument('--local_test', help='if this term is set True and train_holdout>0, (0.5*train_holdout) of data will be set as client.test_data.', action="store_true", default=False)
     # realistic machine config
     parser.add_argument('--seed', help='seed for random initialization;', type=int, default=0)
+    parser.add_argument('--dataseed', help='seed for random initialization for data train/val/test partition', type=int, default=0)
     parser.add_argument('--gpu', nargs='*', help='GPU IDs and empty input is equal to using CPU', type=int)
-    parser.add_argument('--server_with_cpu', help='seed for random initialization;', action="store_true", default=False)
+    parser.add_argument('--server_with_cpu', help='the model parameters will be stored in the memory if True', action="store_true", default=False)
     parser.add_argument('--num_parallels', help="the number of parallels in the clients computing session", type=int, default=1)
     parser.add_argument('--num_workers', help='the number of workers of DataLoader', type=int, default=0)
     parser.add_argument('--pin_memory', help='pin_memory of DataLoader', action="store_true", default=False)
@@ -161,6 +168,17 @@ def read_option_from_command():
             option[key]=[]
     return option
 
+def option_helper():
+    from flgo.utils import option_desc
+    import prettytable as pt
+    lines = option_desc.split('\n')
+    lines = [l.split(',') for l in lines if len(l)>0]
+    tb = pt.PrettyTable(lines[0])
+    for i in range(1, len(lines)):
+        tb.add_row(lines[i])
+    print(tb)
+    return
+
 def load_configuration(config={}):
     r"""
     Load configurations from .yml file or dict.
@@ -180,7 +198,7 @@ def load_configuration(config={}):
     else:
         raise TypeError('The input config should be either a dict or a filename.')
 
-def gen_benchmark_from_file(benchmark:str, config_file:str, target_path='.',data_type:str='cv', task_type:str='classification'):
+def gen_benchmark_from_file(benchmark:str, config_file:str, target_path='.',data_type:str='cv', task_type:str='classification', overwrite:bool=False) -> str:
     r"""
         Create customized benchmarks from configurations. The configuration is a .py file that describes the datasets and the model,
         where there must exist a function named `get_model` and a variable `train_data`. `val_data` and test_data are two optional
@@ -191,13 +209,18 @@ def gen_benchmark_from_file(benchmark:str, config_file:str, target_path='.',data
         target_path: (str): the path to store the benchmark
         data_type (str): the type of dataset that should be in the list ['cv', 'nlp', 'graph', 'rec', 'series', 'tabular']
         task_type (str): the type of the task (e.g. classification, regression...)
+        overwrite (bool): overwrite current benchmark if there already exists a benchmark of the same name
     Returns:
         bmk_module (str): the module name of the generated benchmark
     """
     if not os.path.exists(config_file): raise FileNotFoundError('File {} not found.'.format(config_file))
     target_path = os.path.abspath(target_path)
     bmk_path = os.path.join(target_path, benchmark)
-    if os.path.exists(bmk_path): raise FileExistsError('Benchmark {} already exists'.format(bmk_path))
+    if os.path.exists(bmk_path):
+        if not overwrite:
+            warnings.warn('There already exists a benchmark `{}`'.format(benchmark))
+            return '.'.join(os.path.relpath(bmk_path, os.getcwd()).split(os.path.sep))
+        # raise FileExistsError('Benchmark {} already exists'.format(bmk_path))
     temp_path = os.path.join(flgo.benchmark.path, 'toolkits', data_type, task_type, 'temp')
     if not os.path.exists(temp_path):
         raise NotImplementedError('There is no support to automatically generation of {}.{}. More other types are comming soon...'.format(data_type, task_type))
@@ -288,85 +311,7 @@ def gen_hierarchical_benchmark(benchmark:str, config_file:str, target_path = '.'
     bmk_module = '.'.join(os.path.relpath(bmk_path, os.getcwd()).split(os.path.sep))
     return bmk_module
 
-def gen_task_from_para(benchmark, bmk_para:dict={}, Partitioner=None, par_para:dict={}, task_path: str= '', rawdata_path:str= '', seed:int=0):
-    r"""
-    Generate a federated task according to the parameters of this function. The formats and meanings of the inputs are listed as below:
-
-    Args:
-        benchmark (package|str): the benchmark package or the module path of it
-        bmk_para (dict): the customized parameter dict of the method TaskGenerator.__init__() of the benchmark
-        Partitioner (flgo.benchmark.toolkits.partition.BasicPartitioner|str): the class of the Partitioner or the name of the Partitioner that was realized in flgo.benchmark.toolkits.partition
-        par_para (dict): the customized parameter dict of the method Partitioner.__init__()
-        task_path (str): the path to store the generated task
-        rawdata_path (str): where the raw data will be downloaded\stored
-        seed (int): the random seed used to generate the task
-
-    Example:
-    ```python
-        >>> import flgo
-        >>> import flgo.benchmark.mnist_classification as mnist
-        >>> from flgo.benchmark.toolkits.partition import IIDPartitioner
-        >>> # GENERATE TASK BY PASSING THE MODULE OF BENCHMARK AND THE CLASS OF THE PARTITIOENR
-        >>> flgo.gen_task_from_para(benchmark=mnist, Partitioner = IIDPartitioner, par_para={'num_clients':100}, task_path='./mnist_gen_by_para1')
-        >>> # GENERATE THE SAME TASK BY PASSING THE STRING
-        >>> flgo.gen_task_from_para(benchmark='flgo.benchmark.mnist_classification', Partitioner='IIDPartitioner', par_para={'num_clients':100}, task_path='./mnist_gen_by_para2')
-    ```
-    """
-    random.seed(3 + seed)
-    np.random.seed(97 + seed)
-    torch.manual_seed(12+seed)
-    os.environ['PYTHONHASHSEED'] = str(seed)
-    if type(benchmark) is str: benchmark = importlib.import_module(benchmark)
-    if not hasattr(benchmark, '__path__'): raise RuntimeError("benchmark should be a package or the path of a package")
-    if Partitioner is not None:
-        if type(Partitioner) is str:
-            if Partitioner in globals().keys(): Partitioner = eval(Partitioner)
-            else: Partitioner = getattr(flgo.benchmark.toolkits.partition, Partitioner)
-        partitioner = Partitioner(**par_para)
-    else:
-        try:
-            if hasattr(benchmark, 'default_partitioner'):
-                Partitioner = getattr(benchmark, 'default_partitioner')
-                default_partition_para = getattr(benchmark, 'default_partition_para') if hasattr(benchmark, 'default_partition_para') else {}
-                partitioner = Partitioner(**default_partition_para)
-            else:
-                partitioner = None
-        except:
-            partitioner = None
-    if rawdata_path!='': bmk_para['rawdata_path']=rawdata_path
-    bmk_core = benchmark.core
-    task_generator = getattr(bmk_core, 'TaskGenerator')(**bmk_para)
-    if partitioner is not None:
-        task_generator.register_partitioner(partitioner)
-        partitioner.register_generator(task_generator)
-    task_generator.generate()
-    # save the generated federated benchmark
-    # initialize task pipe
-    if task_path=='': task_path = os.path.join('.', task_generator.task_name)
-    task_pipe = getattr(bmk_core, 'TaskPipe')(task_path)
-    # check if task already exists
-    if task_pipe.task_exists():
-        raise FileExistsError('Task {} already exists.'.format(task_path))
-    try:
-        # create task architecture
-        task_pipe.create_task_architecture()
-        # save meta infomation
-        task_pipe.save_info(task_generator)
-        # save task
-        task_pipe.save_task(task_generator)
-        print('Task {} has been successfully generated.'.format(task_pipe.task_path))
-    except Exception as e:
-        print(e)
-        task_pipe.remove_task()
-        print("Failed to saving splited dataset.")
-    # save visualization
-    try:
-        visualize_func = getattr(benchmark,'visualize')
-        visualize_func(task_generator, partitioner, task_path)
-    except:
-        pass
-
-def gen_task(config={}, task_path:str= '', rawdata_path:str= '', seed:int=0):
+def gen_task(config={}, task_path:str= '', rawdata_path:str= '', seed:int=0, overwrite:bool=False):
     r"""
     Generate a federated task that is specified by the benchmark information and the partition information, where the generated task will be stored in the task_path and the raw data will be downloaded into the rawdata_path.
 
@@ -392,13 +337,6 @@ def gen_task(config={}, task_path:str= '', rawdata_path:str= '', seed:int=0):
     gen_option = load_configuration(config)
     if type(gen_option['benchmark']) is not dict: gen_option['benchmark']={'name':gen_option['benchmark']}
     if 'para' not in gen_option['benchmark'].keys(): gen_option['benchmark']['para'] = {}
-    if 'partitioner' in gen_option.keys():
-        # update parameters of partitioner
-        if 'para' not in gen_option['partitioner'].keys():
-            gen_option['partitioner']['para'] = {}
-        else:
-            if 'name' not in gen_option['partitioner'].keys():
-                gen_option['benchmark']['para'].update(gen_option['partitioner']['para'])
     # init generator
     if rawdata_path!='': gen_option['benchmark']['para']['rawdata_path']=rawdata_path
     if type(gen_option['benchmark']['name']) is str:
@@ -411,35 +349,53 @@ def gen_task(config={}, task_path:str= '', rawdata_path:str= '', seed:int=0):
     bmk_module = importlib.import_module(gen_option['benchmark']['name']) if type(
         gen_option['benchmark']['name']) is str else gen_option['benchmark']['name']
     # create partitioner for generator if specified
-    if 'partitioner' in gen_option.keys() and 'name' in gen_option['partitioner'].keys():
-        Partitioner = gen_option['partitioner']['name']
-        if type(Partitioner) is str:
-            if Partitioner in globals().keys(): Partitioner = eval(Partitioner)
-            else: Partitioner = getattr(flgo.benchmark.partition, Partitioner)
-        partitioner = Partitioner(**gen_option['partitioner']['para'])
-        task_generator.register_partitioner(partitioner)
-        partitioner.register_generator(task_generator)
-    else:
-        try:
-            if hasattr(bmk_module, 'default_partitioner'):
-                Partitioner = getattr(bmk_module, 'default_partitioner')
-                default_partition_para = getattr(bmk_module, 'default_partition_para') if hasattr(bmk_module, 'default_partition_para') else {}
-                partitioner = Partitioner(**default_partition_para)
+    if 'partitioner' in gen_option.keys():
+        if isinstance(gen_option['partitioner'], flgo.benchmark.partition.BasicPartitioner):
+            partitioner = gen_option['partitioner']
+            task_generator.register_partitioner(partitioner)
+            partitioner.register_generator(task_generator)
+        else:
+            if not isinstance(gen_option['partitioner'], dict):
+                gen_option['partitioner'] = {'name': gen_option['partitioner'], 'para':{}}
+            # update parameters of partitioner
+            if 'para' not in gen_option['partitioner'].keys():
+                gen_option['partitioner']['para'] = {}
+            else:
+                if 'name' not in gen_option['partitioner'].keys():
+                    gen_option['benchmark']['para'].update(gen_option['partitioner']['para'])
+            if 'name' in gen_option['partitioner'].keys():
+                Partitioner = gen_option['partitioner']['name']
+                if type(Partitioner) is str:
+                    if Partitioner in globals().keys(): Partitioner = eval(Partitioner)
+                    else: Partitioner = getattr(flgo.benchmark.partition, Partitioner)
+                partitioner = Partitioner(**gen_option['partitioner']['para'])
                 task_generator.register_partitioner(partitioner)
                 partitioner.register_generator(task_generator)
             else:
-                partitioner = None
-        except:
-            partitioner = None
-    # generate federated task
-    task_generator.generate()
-    # save the generated federated benchmark
+                try:
+                    if hasattr(bmk_module, 'default_partitioner'):
+                        Partitioner = getattr(bmk_module, 'default_partitioner')
+                        default_partition_para = getattr(bmk_module, 'default_partition_para') if hasattr(bmk_module, 'default_partition_para') else {}
+                        partitioner = Partitioner(**default_partition_para)
+                        task_generator.register_partitioner(partitioner)
+                        partitioner.register_generator(task_generator)
+                    else:
+                        partitioner = None
+                except:
+                    partitioner = None
     # initialize task pipe
-    if task_path=='': task_path = os.path.join('.', task_generator.task_name)
+    if len(task_path) == 0: task_path = 'FLGoTask_' + uuid.uuid4().hex
     task_pipe = getattr(bmk_core, 'TaskPipe')(task_path)
     # check if task already exists
     if task_pipe.task_exists():
-        raise FileExistsError('Task {} already exists.'.format(task_path))
+        if not overwrite:
+            warnings.warn('Task {} already exists. To overwrite the existing task, use flgo.gen_task(...,overwrite=True,...)'.format(task_path))
+            return
+        else:
+            shutil.rmtree(task_path)
+    # generate federated task
+    task_generator.generate()
+    # save the generated federated benchmark
     try:
         # create task architecture
         task_pipe.create_task_architecture()
@@ -452,13 +408,75 @@ def gen_task(config={}, task_path:str= '', rawdata_path:str= '', seed:int=0):
         print(e)
         task_pipe.remove_task()
         print("Failed to saving splited dataset.")
+        return None
     # save visualization
-    if hasattr(bmk_module, 'visualize'):
-        try:
-            visualize_func = getattr(bmk_module,'visualize')
-            visualize_func(task_generator, partitioner, task_path)
-        except Exception as e:
-            print('Warning: Failed to visualize the partitioned result where there exists error {}'.format(e))
+    try:
+        visualize_func = getattr(bmk_module,'visualize')
+        visualize_func(task_generator, partitioner, task_path)
+    except Exception as e:
+        print('Warning: Failed to visualize the partitioned result where there exists error {}'.format(e))
+    finally:
+        return task_path
+
+def gen_task_by_(benchmark, partitioner:flgo.benchmark.partition.BasicPartitioner=None, task_path:str='', seed:int=0, overwrite:bool=False):
+    """
+    Generate federated task from benchmark and partitioner without inputing other parameters
+    Args:
+        benchmark (module): benchmark
+        partitioner (flgo.benchmark.partition.BasicPartitioner): a instance of type flgo.benchmark.partition.BasicPartitioner
+        task_path (str): the name and the path of the task
+        seed (int): random seed
+        overwrite (bool): overwrite the old task if the task_path already exist if True
+
+    Returns:
+        task_path (str): the path of the task
+    """
+    # generate the name of task randomly if empty
+    if len(task_path)==0: task_path = 'FLGoTask_'+uuid.uuid4().hex
+    # setup random seed
+    random.seed(3 + seed)
+    np.random.seed(97 + seed)
+    torch.manual_seed(12 + seed)
+    os.environ['PYTHONHASHSEED'] = str(seed)
+    bmk_core = importlib.import_module('.core', benchmark.__name__)
+    # load configuration
+    task_generator = getattr(bmk_core, 'TaskGenerator')()
+    if partitioner is not None:
+        task_generator.register_partitioner(partitioner)
+        partitioner.register_generator(task_generator)
+    # check if task already exists
+    task_pipe = getattr(bmk_core, 'TaskPipe')(task_path)
+    if task_pipe.task_exists():
+        if not overwrite:
+            warnings.warn('Task {} already exists. To overwrite the existing task, use flgo.gen_task_by_(...,overwrite=True,...)'.format(task_path))
+            return
+        else:
+            shutil.rmtree(task_path)
+    # generate federated task
+    task_generator.generate()
+    # save the generated federated benchmark
+    # initialize task pipe
+    try:
+        # create task architecture
+        task_pipe.create_task_architecture()
+        # save meta infomation
+        task_pipe.save_info(task_generator)
+        # save task
+        task_pipe.save_task(task_generator)
+        print('Task {} has been successfully generated.'.format(task_pipe.task_path))
+    except Exception as e:
+        print(e)
+        task_pipe.remove_task()
+        print("Failed to saving splited dataset.")
+        return None
+    # save visualization
+    try:
+        visualize_func = getattr(benchmark, 'visualize')
+        visualize_func(task_generator, partitioner, task_path)
+    except Exception as e:
+        print('Warning: Failed to visualize the partitioned result where there exists error {}'.format(e))
+    finally:
+        return task_path
 
 def init(task: str, algorithm, option = {}, model=None, Logger: flgo.experiment.logger.BasicLogger = None, Simulator: BasicSimulator=flgo.simulator.DefaultSimulator, scene='horizontal'):
     r"""
@@ -520,10 +538,12 @@ def init(task: str, algorithm, option = {}, model=None, Logger: flgo.experiment.
     benchmark = task_info['benchmark']
     if model== None:
         bmk_module = importlib.import_module(benchmark)
-        if hasattr(bmk_module, 'default_model'):
+        if hasattr(algorithm, 'init_global_module') or hasattr(algorithm, 'init_local_module'):
+            model = algorithm
+        elif hasattr(bmk_module, 'default_model'):
             model = getattr(bmk_module, 'default_model')
         else:
-            model = algorithm
+            raise NotImplementedError("Model cannot be None when there exists no default model for the current benchmark {} and the algorithm {} didn't define the model by `init_local_module` or `init_global_module`".format(task_info['benchmark'], option['algorithm']))
     option['model'] = (model.__name__).split('.')[-1]
     # create global variable
     gv = GlobalVariable()
@@ -549,9 +569,10 @@ def init(task: str, algorithm, option = {}, model=None, Logger: flgo.experiment.
     core_module = '.'.join([benchmark, 'core'])
     gv.TaskPipe = getattr(importlib.import_module(core_module), 'TaskPipe')
     task_pipe = gv.TaskPipe(task)
-    gv.TaskCalculator = getattr(importlib.import_module(core_module), 'TaskCalculator')
+    TaskCalculator = getattr(importlib.import_module(core_module), 'TaskCalculator')
+    gv.TaskCalculator = TaskCalculator
+    setup_seed(option['dataseed'])
     task_data = task_pipe.load_data(option)
-
     # init objects
     obj_class = [c for c in dir(algorithm) if not c.startswith('__')]
     tmp = []
@@ -559,6 +580,7 @@ def init(task: str, algorithm, option = {}, model=None, Logger: flgo.experiment.
         try:
             C = getattr(algorithm, c)
             setattr(C, 'gv', gv)
+            setattr(C, 'TaskCalculator', TaskCalculator)
             tmp.append(c)
         except:
             continue
@@ -567,16 +589,12 @@ def init(task: str, algorithm, option = {}, model=None, Logger: flgo.experiment.
         for c in obj_class:
             if 'Client' in c:
                 class_client = getattr(algorithm, c)
-                if option['completeness']!='IDL':
-                    class_client.train = flgo.simulator.base.with_completeness(class_client.train)
+                class_client.train = flgo.simulator.base.with_completeness(class_client.train)
             elif 'Server' in c:
                 class_server = getattr(algorithm, c)
-                if option['availability']!='IDL':
-                    class_server.sample = flgo.simulator.base.with_availability(class_server.sample)
-                if option['responsiveness']!='IDL':
-                    class_server.communicate_with = flgo.simulator.base.with_latency(class_server.communicate_with)
-                if option['connectivity']!='IDL':
-                    class_server.communicate = flgo.simulator.base.with_dropout(class_server.communicate)
+                class_server.sample = flgo.simulator.base.with_availability(class_server.sample)
+                class_server.communicate_with = flgo.simulator.base.with_latency(class_server.communicate_with)
+                class_server.communicate = flgo.simulator.base.with_dropout(class_server.communicate)
     objects = task_pipe.generate_objects(option, algorithm, scene=scene)
     obj_classes = collections.defaultdict(int)
     for obj in objects: obj_classes[obj.__class__]+=1
@@ -585,7 +603,6 @@ def init(task: str, algorithm, option = {}, model=None, Logger: flgo.experiment.
     creating_str = ', '.join(creating_str)
     gv.logger.info('SCENE:\t\t{} FL with '.format(scene)+creating_str)
     task_pipe.distribute(task_data, objects)
-
     # init model
     if hasattr(model, 'init_local_module'):
         for object in objects:
@@ -596,7 +613,7 @@ def init(task: str, algorithm, option = {}, model=None, Logger: flgo.experiment.
     if hasattr(model, 'init_dataset'):
         for object in objects:
             model.init_dataset(object)
-
+    setup_seed(option['seed']+346)
     # init communicator
     gv.communicator = flgo.VirtualCommunicator(objects)
 
@@ -608,8 +625,11 @@ def init(task: str, algorithm, option = {}, model=None, Logger: flgo.experiment.
 
     gv.clock = flgo.simulator.base.ElemClock()
     gv.simulator = Simulator(objects, option) if scene == 'horizontal' else None
+    if gv.simulator is not None: gv.simulator.initialize()
     gv.clock.register_simulator(simulator=gv.simulator)
-    gv.logger.register_variable(coordinator=objects[0], participants=objects[1:], option=option, clock=gv.clock, scene=scene, objects = objects)
+    gv.logger.register_variable(coordinator=objects[0], participants=objects[1:], option=option, clock=gv.clock, scene=scene, objects = objects, simulator=Simulator.__name__ if scene == 'horizontal' else 'None')
+    if scene=='horizontal':
+        gv.logger.register_variable(server=objects[0], clients=objects[1:])
     gv.logger.initialize()
     gv.logger.info('Ready to start.')
 
@@ -939,7 +959,7 @@ def multi_init_and_run(runner_args:list, devices = [], scheduler=None):
         res.append(rec)
     return res
 
-def convert_model(get_model:Callable, model_name='anonymous_model', scene:str='horizontal'):
+def convert_model(get_model:Callable, model_name='anonymous', scene:str='horizontal'):
     r"""
     Convert an existing model into a model that can be loaded in flgo.
     Args:
@@ -986,3 +1006,116 @@ def convert_model(get_model:Callable, model_name='anonymous_model', scene:str='h
     else:
         raise NotImplementedError('The current version only support converting model for horizontalFL and DecentralizedFL.')
     return AnonymousModel()
+
+def module2fmodule(Model):
+    """
+    Convert a class of torch.nn.Module into class flgo.utils.fmodule.FModule
+    Args:
+        Model (class): a class inherited from torch.nn.Module
+
+    Returns:
+        TempModule (class): The same class but additionally inheriting from flgo.utils.fmodule.FModule
+
+    """
+    class TempFModule(Model, flgo.utils.fmodule.FModule):
+        def __init__(self, *args, **kwargs):
+            super(TempFModule, self).__init__(*args, **kwargs)
+    return TempFModule
+
+def set_data_root(data_root:str=None):
+    """
+    Set the root of data that stores all the raw data automatically
+    Args:
+        data_root (str): the path of a directory whose default value is None and will be set to os.getcwd() as default.
+    Returns:
+    """
+    file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'benchmark', '__init__.py')
+    default_root = os.path.abspath(os.path.join(flgo.benchmark.path, 'RAW_DATA'))
+    if data_root is None and os.path.abspath(flgo.benchmark.data_root)!=default_root:
+        crt_root = default_root.strip()
+        root_name = '"'+default_root.strip()+'"'
+    elif data_root == 'cwd':
+        crt_root = os.path.abspath(os.getcwd())
+        root_name = 'os.getcwd()'
+    else:
+        if not os.path.exists(data_root):
+            os.makedirs(data_root)
+        if not os.path.isdir(data_root):
+            raise TypeError('data_root must be a dir')
+        crt_root = os.path.abspath(data_root).strip()
+        root_name = '"'+crt_root+'"'
+    with open(file_path, 'r', encoding=sys.getfilesystemencoding()) as inf:
+        lines = inf.readlines()
+        idx = -1
+        for i,line in enumerate(lines):
+            if line.find('data_root')>-1:
+                idx = i
+                break
+        if idx>0:
+            lines[idx] = "data_root = "+ root_name
+    with open(file_path, 'w', encoding=sys.getfilesystemencoding()) as outf:
+        outf.writelines(lines)
+    flgo.benchmark.data_root = crt_root
+    print('Data root directory has successfully been changed to {}'.format(crt_root))
+    return
+
+def download_resource(root:str, name:str, type:str, overwrite:bool=False):
+    """
+    Download resource from github
+    Args:
+        root (str): the path to store the resource
+        name (str): the name of the resource
+        type (type): the type of the resource in ['algorithm', 'benchmark', 'simulator']
+        overwrite (bool): whether to overwrite existing file
+    Returns:
+    """
+    resource_root = "https://github.com/WwZzz/easyFL/raw/FLGo/resources/"
+    if type not in ['algorithm', 'benchmark', 'simulator']: raise ValueError("Args type must of value in ['algorithm', 'benchmark', 'simulator']")
+    url = resource_root+type+'/'
+    suffix_dict = {'algorithm':'.py', 'simulator':'.py', 'benchmark':'.zip'}
+    suffix = suffix_dict[type]
+    file_name = name
+    if not file_name.endswith(suffix):
+        file_name = file_name+suffix
+    if not os.path.exists(file_name) or overwrite:
+        try:
+            urllib.request.urlretrieve(url+file_name, os.path.join(root, file_name))
+        except Exception as e:
+            print(e)
+            return None
+    else:
+        warnings.warn("There already exist {} named {}".format(type, name))
+    if type == 'benchmark':
+        bmk_zip = zipfile.ZipFile(file_name)
+        bmk_zip.extractall(root)
+    module_path = '.'.join(os.path.relpath(os.path.join(root, name), os.path.curdir).split(os.path.sep))
+    module = importlib.import_module(module_path)
+    if type in ['algorithm', 'benchmark']:
+        return module
+    else:
+        Simulator = getattr(module, 'Simulator')
+        return Simulator
+
+def list_resource(type:str='algorithm'):
+    """
+    List currently available resources at github. The arg. `type` should be one of elements in {'algorithm', 'benchmark', 'simulator'}
+    Args:
+        type (str):
+    Returns:
+        res (list): the name of currently available resources
+    """
+    if type not in ['algorithm', 'benchmark', 'simulator']: raise ValueError("Args type must of value in ['algorithm', 'benchmark', 'simulator']")
+    url = "https://github.com/WwZzz/easyFL/tree/FLGo/resources/"+type
+    suffix_dict = {"algorithm": ".py", "benchmark":".zip", "simulator":".py"}
+    suffix = suffix_dict[type]
+    try:
+        content = str(requests.get(url).content, encoding=sys.getfilesystemencoding())
+    except Exception as e:
+        print(e)
+        return None
+    res = re.findall(r'"[a-zA-Z0-9_-]*{}"'.format(suffix), content)
+    res = [s.strip('"') for s in res]
+    res = [s[:-len(suffix)] for s in res if s!="test.py"]
+    return res
+
+
