@@ -145,6 +145,17 @@ class FModule(nn.Module):
         self.op_without_graph()
         self.load_state_dict((self/(self**2)).state_dict())
 
+    def has_nan(self):
+        r"""
+        Check whether there is nan value in model's parameters
+        Returns:
+            res (bool): True if there is nan value
+        """
+        for p in self.parameters():
+            if torch.any(torch.isnan(p)).item():
+                return True
+        return False
+
     def get_device(self):
         r"""
         Returns:
@@ -161,23 +172,18 @@ class FModule(nn.Module):
         Returns:
             the number of all the parameters in this model
         """
-        try:
-            import prettytable as pt
-        except:
-            print('Please install prettytable through `pip install prettytable` before calling this func')
-            return
-        table = pt.PrettyTable(["Modules", "Parameters"])
+        # table = pt.PrettyTable(["Modules", "Parameters"])
         total_params = 0
         for name, parameter in self.named_parameters():
             if not parameter.requires_grad:
-                table.add_row([name, 0])
+                # table.add_row([name, 0])
                 continue
             params = parameter.numel()
-            table.add_row([name, params])
+            # table.add_row([name, params])
             total_params += params
-        if output:
-            print(table)
-            print(f"TotalTrainableParams: {total_params}")
+        # if output:
+        #     print(table)
+        #     print(f"TotalTrainableParams: {total_params}")
         return total_params
 
 def normalize(m):
@@ -852,7 +858,7 @@ def _modeldict_cossim(md1, md2):
     l1 = torch.tensor(0.).to(md1[list(md1)[0]].device)
     l2 = torch.tensor(0.).to(md1[list(md1)[0]].device)
     for layer in md1.keys():
-        if md1[layer] is None or md1[layer].requires_grad==False:
+        if md1[layer] is None:
             continue
         res += (md1[layer].view(-1).dot(md2[layer].view(-1)))
         l1 += torch.sum(torch.pow(md1[layer], 2))
